/**
* Script: amble.js
* Written by: Radnen
* Updated: 12/15/2013
**/

function CMD(cmd) {
	switch (cmd) {
		case NORTH: return "north";
		case SOUTH: return "south";
		case EAST: return "east";
		case WEST: return "west";
	}
	return "nil";
}

var amble = (function(cm, me, dp) {
	var queue = [];
	var input = "";
	var faced = false;
	var blocked = {};
	var init = false;
	var size = 16;
	
	ChangeMap = function(map) {
		blocked = {};
		cm(map);
		init = true;
	}
	
	MapEngine = function(name, fps) {
		blocked = {};
		init = true;
		me(name, fps);
	}
	
	DestroyPerson = function(name) {
		if (name in blocked) delete blocked[name];
		dp(name);
	}
		
	function Block(x, y, name) {
		blocked[name] = { x: x, y: y };
	}
	
	function IsObstructed(x, y) {
		for (var i in blocked) {
			if (!blocked[i]) continue;
			if (x == blocked[i].x && y == blocked[i].y) return true;
		}
		return false;
	}

	function IsBlocked(name, cmd) {
		var v = GetHeading(cmd);
		var x = Math.floor(GetPersonX(name) / size) + v.x;
		var y = Math.floor(GetPersonY(name) / size) + v.y;
		for (var i in blocked) {
			if (i == name) continue;
			if (!blocked[i]) continue;
			if (x == blocked[i].x && y == blocked[i].y) return true;
		}
		return IsPersonObstructed(name, x * size + 7, y * size + 7);
	}
	
	function Enqueue(name, cmd) {
		queue.push({name: name, cmd: cmd});
	}
	
	function Step(name, cmd) {
		var face = GetFace(cmd);
		
		QueuePersonCommand(name, face, true);
		if (name == input && !faced) {
			for (var i = 0; i < 4; ++i) Queue(input, COMMAND_WAIT, 1);
			faced = true;
			return;
		}
		
		if (!IsBlocked(name, cmd)) {
			var v = GetHeading(cmd);
			var tx = GetTileX(name);
			var ty = GetTileY(name);
			amble.block(tx + v.x, ty + v.y, name);
			QueuePersonCommand(name, face, true);
			for (var i = 0; i < size / GetPersonSpeedX(name); ++i) {
				QueuePersonCommand(name, cmd, false);
			}
			//QueuePersonScript(name, "TalkUpdater('" + name + "');", true);
		}
		else if (name == input) {
			Queue("player", COMMAND_ANIMATE, 16);
			QueuePersonScript("player", "Audio.playSound(Resources.sounds.wall);", true);
		}
	}
	
	function GetDirectionVector(name) {
		var v = { x: 0, y: 0 };
		
		switch (GetPersonDirection(name)) {
			case "north": v.y = -1; break;
			case "south": v.y = 1; break;
			case "east": v.x = 1; break;
			case "west": v.x = -1; break;
		}
		
		return v;
	}

	function GetHeading(cmd) {
		var v = { x: 0, y: 0 };
		
		switch (cmd) {
			case NORTH: v.y = -1; break;
			case SOUTH: v.y = 1; break;
			case EAST: v.x = 1; break;
			case WEST: v.x = -1; break;
		}
		
		return v;
	}
	
	function GetFace(cmd) {
		var face = COMMAND_FACE_NORTH;
		switch (cmd) {
			case SOUTH: face = COMMAND_FACE_SOUTH; break;
			case EAST: face = COMMAND_FACE_EAST; break;
			case WEST: face = COMMAND_FACE_WEST; break;
		}
		return face;
	}
	
	function AttachInput(name) { input = name; }
	function DetachInput() { input = ""; }
	function GetInput() { return input; }
	
	function Update() {
		if (input != "") UpdateInput();
		
		for (var i = 0; i < queue.length; ++i) {
			if (!DoesPersonExist(queue[i].name)) { queue.splice(i, 1); i--; }
			else if (IsCommandQueueEmpty(queue[i].name)) {
				Step(queue[i].name, queue[i].cmd);
				queue.splice(i, 1);
				i--;
			}
		}
		
		if (init) {
			List.foreach(GetPersonList(), function(name) {
				Block(GetTileX(name), GetTileY(name), name);
			});
			init = false;
		}
	}
	
	function Render() {
		for (var i in blocked) {
			var b = blocked[i];
			if (!b) continue;
			Rectangle(MapToScreenX(0, b.x * size), MapToScreenY(0, b.y * size), size, size, red);
			Resources.fonts.font.drawText(MapToScreenX(0, b.x * size), MapToScreenY(0, b.y * size), i);
		}
	}
	
	function UpdateInput() {
		if (!IsCommandQueueEmpty(input)) return;

		var x = GetPersonX(input);
		var y = GetPersonY(input);
		
		if (x % size != 0)
			SetPersonX(input, Math.floor(x / size) * size + 7);
		if (y % size != 0)
			SetPersonY(input, Math.floor(y / size) * size + 7);

		var tile = GetTile(x / GetTileWidth(), y / GetTileHeight(), GetPersonLayer(input));
		var name = GetTileName(tile), s = 1;
		if (name == "slow") s = 0.75;
		if (name == "fast") s = 1.5;
		SetPersonSpeed(input, s);
		
		if (IsKeyPressed(KEY_UP) || Input.axisUp) {
			Enqueue(input, NORTH);
		}
		else if (IsKeyPressed(KEY_DOWN) || Input.axisDown) {
			Enqueue(input, SOUTH);
		}
		else if (IsKeyPressed(KEY_LEFT) || Input.axisLeft) {
			Enqueue(input, WEST);
		}
		else if (IsKeyPressed(KEY_RIGHT) || Input.axisRight) {
			Enqueue(input, EAST);
		}

		if (faced && !IsKeysPressed(KEY_UP, KEY_DOWN, KEY_LEFT, KEY_RIGHT) && !IsAnyAxis()) faced = false;
	}
	
	function IsAnyAxis() {
		return Input.axisLeft || Input.axisRight || Input.axisUp || Input.axisDown;
	}
	
	return {
		update: Update,
		render: Render,
		enqueue: Enqueue,
		attachInput: AttachInput,
		detachInput: DetachInput,
		getHeading: GetHeading,
		getDirectionVector: GetDirectionVector,
		getInput: GetInput,
		isBlocked: IsBlocked,
		isObstructed: IsObstructed,
		
		// internal use:
		block: Block,
	};
})(ChangeMap, MapEngine, DestroyPerson);
