/**
* Script: binarytree.js
* Written by: Radnen
* Updated: 12/16/2013
**/

function TreeNode(object)
{
	this.data = object;
	this.left = null;
	this.right = null;
}

function BinaryTree()
{
	this.root = null;
	this.compare = function(a, b) { return a > b; }
	this.equals = function(a, b) { return a == b; }
	this.rank = 0;
}

/**
* - this clears the tree for reuse.
**/
BinaryTree.prototype.clear = function()
{
	this.root = null;
}

/**
* - used for debug purposes, returns a human readable string of the contents.
**/
BinaryTree.prototype.toAString = function(data) {
	if (!data) return "{}";
	return data.data + " {l: " + this.toVisual(data.left) + ", r: " + this.toVisual(data.right) + "}";
}

/**
* - adds a node containing the data element sorted by the compare function.
**/
BinaryTree.prototype.add = function(data)
{
	if (!this.root) { this.root = new TreeNode(data); return; }
	
	var current = this.root;
	var previous = null;
	var left = false;
	
	while (current) {
		previous = current;
		left = this.compare(current.data, data);
		if (left) current = current.left;
		else current = current.right;
	}
	
	if (left) previous.left = new TreeNode(data);
	else previous.right = new TreeNode(data);
}

/**
* - returns true if the data element is indeed within the tree.
**/
BinaryTree.prototype.contains = function(data) {
	var temp = this.root;
	var rank = 0;
	
	while (temp) {
		if (temp.data == data) return true;
		else if (this.compare(temp.data, data))
			temp = temp.left;
		else
			temp = temp.right;
		rank++;
	}
	
	if (rank > this.rank) this.rank = rank;
	
	return false;
}

/**
* - shift returns the left-most data in the tree.
**/
BinaryTree.prototype.shift = function() {
	var prev, temp = this.root;
	
	while (temp.left) {
		prev = temp;
		temp = temp.left;
	}
	
	if (!prev) {
		var t = this.root.data;
		this.root = this.root.right;
		return t;
	}
	
	if (temp.right) prev.left = temp.right;	
	else prev.left = null;
	
	return temp.data;
}

/**
* - pop returns the right-most data in the tree.
**/

/**
* - remove searches for and removes a node containing the data element.
**/
BinaryTree.prototype.remove = function(data) {
	var temp = this.root;
	while (temp) {
		if (this.equals(temp.data, data)) {
			temp = this.removeAt(temp);
		}
		else if (this.compare(temp.data, data))
			temp = temp.left;
		else
			temp = temp.right;
	}
	
}

/**
* - internal use, removes a node at the specific location.
**/
BinaryTree.prototype.removeAt = function(node) {
	if (!node.left && !node.right) return null;
	else if (!node.left) {
		return node.right;
	}
	else if(!node.right) {
		return node.left;
	}
	else {
		var data = this.getInOrderSuccessor(node);
		node.data = data;
		return node;
	}
}

/**
* - internal use, gets the inorder successor for node removal.
**/
BinaryTree.prototype.getInOrderSuccessor = function(node) {
	var retVal, prev;
	
	while (node.left) {
		prev = node;
		node = node.left;
	}
	Alert(node.data);
	
	prev.left = null;
	return node.data;
}