/**
* Script: main.js
* Written by: Radnen
* Updated: 12/16/2013
**/

const NORTH = COMMAND_MOVE_NORTH;
const SOUTH = COMMAND_MOVE_SOUTH;
const EAST = COMMAND_MOVE_EAST;
const WEST = COMMAND_MOVE_WEST;

const SW = GetScreenWidth();
const SH = GetScreenHeight();

const PI = Math.PI;

RequireScript("enemy.js");
RequireScript("assert.js");
RequireScript("resources.js");
RequireScript("tween.js");
RequireScript("legacy.js");
RequireScript("audio.js");
RequireScript("input.js");
RequireScript("list.js");
RequireScript("query.js");
RequireScript("amble.js");
RequireScript("stuff.js");
RequireScript("traps.js");
RequireScript("items.js");
RequireScript("pathfinder.js");
RequireScript("pathing.js");

Resources.loadAll();

function game()
{
	Audio.musicfolder = "../music";
	Audio.setMaxMusicVolume(180);
	Audio.play("dungeon-v2.ogg");
	Pathfinder.init(53, 35);
	while (true) { ShowTitle(); }
}

var g_imgs = ["bomb", "trap"];
var g_traps = [BombTrap, SpikeTrap];
var g_counts = [0, 0];
var g_cur_type = 0;
var g_score = 0;
var g_name = "";
var g_ss = "player1.rss";

function update() {
	amble.update();
	engine.update();
	
	if (Input.onKeyDown(KEY_SPACE) || Input.onButtonDown(Input.AButton)) {
		engine.swing("player");
	}
	
	if (Input.onKeyDown(KEY_CTRL) || Input.onButtonDown(Input.BButton)) {
		if (g_counts[g_cur_type] > 0) {
			engine.placeTrap(g_traps[g_cur_type], GetTileX("player"), GetTileY("player"));
			g_counts[g_cur_type]--;
		}
	}
	
	if (Input.onButtonDown(Input.LButton)) {
		g_cur_type = (g_cur_type == 0) ? 1 : 0;
	}

	if (Input.onButtonDown(Input.RButton)) {
		g_cur_type = (g_cur_type == 1) ? 0 : 1;
	}
	
	if (Input.onKeyDown(KEY_1)) {
		g_cur_type = 0;
	}

	if (Input.onKeyDown(KEY_2)) {
		g_cur_type = 1;
	}
}

function render() {
	engine.render();
	Resources.images.itemslot.blit(SW/2-10, 2);
	Resources.images[g_imgs[g_cur_type]].blit(SW/2-10, 2);
	Resources.fonts.font.drawText(SW/2, 8, "x"+g_counts[g_cur_type]);
}

function GetTileX(name)
{
	return GetPersonX(name) >> 4;
}

function GetTileY(name)
{
	return GetPersonY(name) >> 4;
}

function Queue(name, dir, times) {
	while (times--) QueuePersonCommand(name, dir, false);
}

function IsKeysPressed() {
	for (var i = 0; i < arguments.length; ++i) {
		if (IsKeyPressed(arguments[i])) return true;
	}
	return false;
}

// Returns an angle between 0 and 2PI, between two entities
function GetAngleBetweenEntities(person1, person2)
{
	var dx = GetPersonX(person1)-GetPersonX(person2);
	var dy = GetPersonY(person1)-GetPersonY(person2);
	
	return Math.atan2(dy, dx)+PI;
}

function GetAngleDirection(person1, person2)
{
	var angle = GetAngleBetweenEntities(person1, person2);
	var d = (angle / PI) * 180;
		
	if (d >= 45 && d < 135) return "north";
	if (d >= 135 && d < 225) return "east";
	if (d >= 225 && d < 315) return "south";
	return "west";
}

function GetDistance(name1, name2) {
	var d1 = GetPersonX(name1) - GetPersonX(name2);
	var d2 = GetPersonY(name1) - GetPersonY(name2);
	return Math.sqrt(d1*d1 + d2*d2);
}

function GetPersonAt(x, y) {
	return Query.select(GetPersonList()).where(function(p) {
			var px = GetTileX(p);
			var py = GetTileY(p);
			return px == x && py == y;
		}).first();
}

function Player() {
	this.maxhp = 50;
	this.hp = 50;
	this.kills = 0;
	this.dead = false;
}

Player.prototype.hurt = function(v) {
	this.hp -= v;
	SetTile(GetTileX("player"), GetTileY("player"), 1, 16);
	if (this.hp <= 0 && !this.dead) { this.dead = true; ShowDeath(); }
}

Player.prototype.heal = function(v) {
	this.hp += v;
	if (this.hp > this.maxhp) this.hp = this.maxhp;
}

var engine = (function() {
	var counter = 0;
	var zombies = [];
	var traps = [];
	var time = GetTime();
	var types = ["zombie.rss", "skelly.rss", "bat.rss"];
	var player = new Player();
	var items = [];
	var start = 0;
	var daytime = 0;
	var win = false;
	
	function Reset() {
		counter = 0;
		zombies = [];
		traps = [];
		items = [];
		time = GetTime();
		player = new Player();
		start = GetTime();
		win = false;

		g_counts = [0, 0];
		g_cur_type = 0;
		g_score = 0;
		g_name = "";
		g_ss = "player1.rss";
	}
	
	function addZombie() {
		if (amble.isObstructed(45, 6) || GetPersonAt(45, 5)) return;
		counter++;
		var name = "zombie_" + counter;
		zombies.push(new Enemy(name));
		CreatePerson(name, Query.select(types).random(), true);
		SetPersonSpeed(name, 0.50);
		SetPersonXYFloat(name, 45*16+7, 5*16+7);

		if (amble.isObstructed(16, 34) || GetPersonAt(16, 35)) return;
		counter++;
		var name = "zombie_" + counter;
		zombies.push(new Enemy(name));
		CreatePerson(name, Query.select(types).random(), true);
		SetPersonSpeed(name, 0.50);
		SetPersonXYFloat(name, 16*16+7, 35*16+7);
	}
	
	function AddItem(type, x, y) {
		var item = new Item(type);
		if (item.create(x, y)) items.push(item);
	}
		
	function PlaceTrap(traptype, x, y) {
		var name = "trap." + x + "." + y;
		if (DoesPersonExist(name)) return;
		var trap = new traptype(name);
		traps.push(trap);
		CreatePerson(name, trap.ss, true);
		SetPersonLayer(name, trap.layer);
		SetPersonXYFloat(name, x*16+7, y*16+7);
	}
	
	function trapai(t) {
		if (t.ai) t.ai();
	}
	
	function dist(x1, y1, x2, y2) {
		var dx = x1 - x2;
		var dy = y1 - y2;
		return Math.sqrt(dx*dx + dy*dy);
	}
	
	function ai(z) {
		z.update();
		if (!IsCommandQueueEmpty(z.name)) return;
		
		var sx = GetTileX(z.name);
		var sy = GetTileY(z.name);
		var px = GetTileX("player");
		var py = GetTileY("player");

		var command = null;
		var a = GetAngleDirection("player", z.name);
		switch (a) {
			case "north": command = NORTH; break;
			case "south": command = SOUTH; break;
			case "east": command = EAST; break;
			case "west": command = WEST; break;
		}
		
		if (amble.isBlocked(z.name, command)) {
			if (command == NORTH) {
				if (dist(sx-1, sy, px, py) < dist(sx+1, sy, px, py)) command = WEST;
				else command = EAST;
			}
			if (command == SOUTH) {
				if (dist(sx-1, sy, px, py) < dist(sx+1, sy, px, py)) command = WEST;
				else command = EAST;
			}
			if (command == EAST) {
				if (dist(sx, sy-1, px, py) < dist(sx, sy+1, px, py)) command = NORTH;
				else command = SOUTH;
			}
			if (command == WEST) {
				if (dist(sx, sy-1, px, py) < dist(sx, sy+1, px, py)) command = NORTH;
				else command = SOUTH;
			}
		}
		
		if (command != null) amble.enqueue(z.name, command);
	}
	
	function GetEnemy(name) {
		return Query.select(zombies).where(function(obj) { return obj.name == name; }).first();
	}
	
	function UpdateAll(array, func) {
		for (var i = 0; i < array.length; ++i) {
			if (array[i].dead) { DestroyPerson(array[i].name); array.splice(i, 1); i--; }
			else func(array[i]);
		}
	}
	
	function Swing(pname) {
		var dir = GetPersonDirection(pname);
		var v = amble.getDirectionVector(pname);
		var ss = "";
		if (dir == "north" || dir == "south") {
			ss = "vertsword.rss";
		}
		if (dir == "west" || dir == "east") {
			ss = "horzsword.rss";
		}
		
		var name = pname + "_sword";
		if (DoesPersonExist(name)) return;
		
		CreatePerson(name, ss, true);
		SetPersonDirection(name, dir);
		SetPersonXYFloat(name, (GetTileX(pname) + v.x)*16 + 7, (GetTileY(pname) + v.y)*16 + 7);
		SetPersonLayer(name, 2);
		Queue(name, COMMAND_ANIMATE, 6);
		QueuePersonScript(name, "engine.checkHit('" + pname + "');", true);
		Queue(name, COMMAND_ANIMATE, 5);
		QueuePersonScript(name, "DestroyPerson(GetCurrentPerson());", true);
	}
	
	function CheckHit(name) {
		if (!DoesPersonExist(name)) return;
		if (name == "player") {
			List.foreach(zombies, function(z) {
				var angle = GetAngleDirection(z.name, "player");
				var dist = GetDistance(z.name, "player");
				if (dist < 40 && angle == GetPersonDirection("player")) z.hurt(10);
			});
		}
		else {
			var angle = GetAngleDirection("player", name);
			var dist = GetDistance(name, "player");
			if (dist < 40 && angle == GetPersonDirection(name))  player.hurt(5);
		}
	}
	
	function updateloop(i) { i.update(); }
	
	function Update() {
		if (player.hp <= 0 || win) return;
		
		UpdateAll(zombies, ai);
		UpdateAll(traps, trapai);
		UpdateAll(items, updateloop);
		
		daytime = 300 - Math.floor((GetTime() - start)/1000);
		if (daytime <= 0) {
			ShowWin();
			win = true;
		}
		
		if (time + 1000 < GetTime() && zombies.length < 12) {
			addZombie();
			time = GetTime();
		}
	}
	
	function Render() {
		List.foreach(zombies, function(z) { z.render(); });
		var min = Math.floor(daytime / 60);
		var secs = daytime - min * 60;
		Resources.fonts.font.drawText(2, 2, "0" + min + ":" + ((secs < 10) ? "0" : "") + secs);
		
		Rectangle(SW-69, 3, 66, 6, black);
		Rectangle(SW-68, 4, (player.hp / player.maxhp) * 64, 4, red);
	}
	
	function GetEnemies() { return zombies; }
	function GetTraps() { return traps; }
	
	return {
		update: Update,
		render: Render,
		placeTrap: PlaceTrap,
		getEnemy: GetEnemy,
		get player() { return player; },
		reset: Reset,
		swing: Swing,
		checkHit: CheckHit,
		getEnemies: GetEnemies,
		getTraps: GetTraps,
		addItem: AddItem,
	};
}());