/**
* Script: pathfinder.js
* Written by: Radnen
* Updated: 12/16/2013
**/

RequireScript("binarytree.js");

function Node(data)
{
	this.data = data;
	this.next = null;
}

function SortedList()
{
	this.root = null;
	this.data = [];
}

SortedList.prototype.clear = function() { this.root = null; this.data = []; }

SortedList.prototype.add = function(data) {
	this.data[data.x + "." + data.y] = true;
	
	var n = new Node(data);

	if (this.root == null) { this.root = n; return; }
	
	var cur = this.root;
	var prev = null;
	
	while (cur) {
		prev = cur;
		cur = cur.next;

		if (cur && data.f < cur.data.f) {
			prev.next = n;
			n.next = cur;
			return;
		}
	}
	
	prev.next = n;
}

SortedList.prototype.contains = function(data) {
	return this.data[data.x + "." + data.y];
}

SortedList.prototype.shift = function() {
	if (this.root == null) return null;
	var n = this.root;
	this.root = this.root.next;
	this.data[n.data.x + "." + n.data.y] = false;
	return n.data;
}

SortedList.prototype.print = function() {
	var p = "";
	var node = this.root;
	while (node) {
		p += node.data + " ";
		node = node.next;
	}
	return p;
}

function PathNode(x, y)
{
	this.x = x;
	this.y = y;
	
	this.f = 0;
	this.g = 0;
	this.h = 0;
	
	this.next = null;
}

var Pathfinder = {
	open: new SortedList(), // binary tree to order opened nodes
	closed: [],				// hash to store closed nodes
		
	grid: [],				// contains nodes
	width: 0,				// width of pathing field
	height: 0,				// height of pathing field
	
	init: function(width, height)
	{
		this.width = width;
		this.height = height;
		
		for (var x = 0, w = 0; x < width; ++x) {
			for (var y = 0; y < height; ++y) {
				this.grid[x*width + y] = new PathNode(x, y);
			}
		}
	},
	
	doPath: function(name, x1, y1, x2, y2)
	{
		this.open.clear();
		this.closed = [];
		var cur, n = 0, w = 0, h = 0, temp, ww;

		temp = this.getStart(x1, y1);		
		this.closed[temp] = true;
		cur = this.grid[temp];
		
		while(true) {
			w = cur.x + 2;
			h = cur.y + 2;
			
			for (var x = cur.x-1; x < w; ++x) {				
				if (x < 0 || x == this.width) continue;
				
				ww = x * this.width;
				
				for (var y = cur.y-1; y < h; ++y)
				{
					if (y < 0 || y == this.height) continue;					
					n = ww + y;
					
					if (this.closed[n]) continue;
					
					temp = this.grid[n];

					if (IsPersonObstructed(name, x*16+7, y*16+7)) continue;
					
					if (!this.open.contains(temp)) {
						if (x == cur.x || y == cur.y) temp.g = cur.g + 10;
						else temp.g = cur.g + 20;
						temp.h = Math.abs(temp.x - x2) + Math.abs(temp.y - y2);
						temp.f = temp.g + temp.h;
						temp.next = cur;
						this.open.add(temp);
					}
					else if (cur.g < temp.g) {
						if (x == cur.x || y == cur.y) temp.g = cur.g + 10;
						else temp.g = cur.g + 20;
						temp.f = temp.g + temp.h;
						temp.next = cur;
					}
				}
			}
			
			if (this.open.root == null) return null;
			
			cur = this.open.shift();
			this.closed[(cur.x*this.width)+cur.y] = true;
			
			if (cur.x == x2 && cur.y == y2) break;
		}		
		
		return this.firstbest(cur);
	},
	
	getStart: function(x, y)
	{
		var i = x*this.width + y;
		var n = this.grid[i];
		n.f = n.g = n.h = 0;
		n.next = null;
		return i;
	},
	
	firstbest: function(node) {
		if (!node) return null;
		if (!node.next) return node;
		while (node.next.next) { node = node.next; }
		return node;
	},
	
	follow: function(node) {
		var arr = [];
				
		while (node) {
			arr.unshift({x: node.x, y: node.y});
			node = node.next;
		}
		
		return arr;
	},
};

Pathfinder.open.compare = function(a, b) {
	return a.f > b.f;
}

/*function game() {
	Pathfinder.init(64, 56);
	var t = GetTime();
	Pathfinder.doPath("", 0, 0, 63, 55);
	Abort(GetTime() - t);
}*/